/*  Copyright (C) MOXA Inc. All rights reserved.

    This software is distributed under the terms of the
    MOXA License.  See the file COPYING-MOXA for details.
*/

/**
 *  \file     EZPage.java
 *  \brief    easy page for device. \n
 *            a example for accesss device by web. \n
 *
 *  2010-03-15	Ryan Ma \n
 *	new release \n
 */
import com.moxa.library.neci.*;
import javax.swing.*;
import javax.swing.tree.*;
import javax.swing.JTree;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.net.*;
import java.util.*;
import java.sql.Time;
import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;
import javax.swing.text.PlainDocument;
import java.util.regex.*;

public class EZPage extends JApplet {
	static public final boolean MULTILANGUAGE = false;			/**< support multi-language */
	static public final boolean TCPSVREXAMPLE = true;			/**< support TCP Server example */
	static public final int  TIMEOUT	= 1000;					/**< default timeout */
	static public final int	 RETRYCOUNT = 3;					/**< default retry count */
	// socket variable
	private int  timeout = TIMEOUT;								/**< the timeout variable */
	private int  retry   = RETRYCOUNT;							/**< the retry count variable */
	// Model index
	static public final int	   UNKNOWN_MODEL = 0;				/**< model index : unknown model */
	static public final int	   MIINEPORT_E2H = 1;				/**< model index : MiiNePort E2-H */
	static public final int	   MIINEPORT_E2  = 2;				/**< model index : MiiNePort E2  */
  static public final int	   MIINEPORT_E3H = 3;				  /**< model index : MiiNePort E3-H */
	static public final int	   MIINEPORT_E3  = 4;				/**< model index : MiiNePort E3  */
	// back index
	static public final int	   NONE_PAGE     = 0;				/**< back index : none page */
	static public final int	   OVERVIEW_PAGE = 1;				/**< back index : overview page */
	static public final int	   BASIC_FOLDER  = 2;				/**< back index : basic folder */
	static public final int	   NETWORK_PAGE  = 3;				/**< back index : network page */
	static public final int	   SERIAL_PAGE   = 4;				/**< back index : serial page */
	static public final int	   OPMODE_PAGE   = 5;				/**< back index : op mode page */
	static public final int	   SAVERST_PAGE  = 6;				/**< back index : save and restart page */
	// main page
	private Container          mainPage;						/**< main page */
	// right page
	private JPanel             rightPage;						/**< right page that show detail configuration items */
	private GridBagLayout      gbLayout;						/**< the layotu of right page */
	private GridBagConstraints containPage;					/**< the Constraint of right page */
	// left tree view
	private DefaultTreeModel model;								/**< the default tree model */
	private JTree tree;											/**< the tree class that show all page */
	private ImageIcon openFolder;								/**< the tree picture when node was selected */
	private ImageIcon closedFolder;								/**< the tree picture when node was not selected */
	// submit
	private ButtonHandler handler = null;						/**< button handler */
	private ComboHandler  comboHandler = null;					/**< combo handler */
	private JButton networkSubmit = null;						/**< submit button of network page */
	private JButton serialSubmit  = null;						/**< submit button of serial page */
	private JButton opmodeSubmit  = null;						/**< submit button of op mode page */
	private JButton back     = null;							/**< back button of save and restart page */
	private JButton restart  = null;							/**< restart button of save and restart page */
	private JButton home     = null;							/**< home button of save and restart page */
	private int     backIdx = NONE_PAGE;						/**< back index */
	// overview
	private JLabel serialNoInfo    = null;					/**< the label that show serial no */
	private JLabel deviceNameInfo  = null;					/**< the label that show device name */
	private JLabel firmwareVerInfo = null;					/**< the label that show firmware version */
	private JLabel ethIpInfo  = null;							/**< the label that show ethernet ip */
	private JLabel ethMacInfo = null;							/**< the label that show ethernet mac */
	private JLabel upTimeInfo = null;							/**< the label that show system up time */
	private JLabel serialComStatusInfo = null;				/**< the label that show serial command status */
	// network
	private JTextField deviceNameInput = null;					/**< the Text Field that show device name */
	private JComboBox  ipCfgInput = null;							/**< the combo box that show ip configuration */
	private String ipCfgStr[]
	= {"Static", "DHCP", "DHCP/BOOTP", "BOOTP", "AUTOIP"};		/**< the string array of ip configuration */
	private JTextField ipInput = null;								/**< the Text Field that show ip address */
	private JTextField netmaskInput  = null;						/**< the Text Field that show netmask */
	private JTextField gatewayInput  = null;						/**< the Text Field that show gateway */
	private JTextField dns1Input     = null;						/**< the Text Field that show dns1 address */
	private JTextField dns2Input     = null;						/**< the Text Field that show dns2 address */
	// serial
	private JTextField portAliasInput  = null;						/**< the Text Field that show port alias */
	private JComboBox  baudRateInput   = null;						/**< the combo box that show baud rate */
	private JTextField linearBaudInput = null;						/**< the Text Field that show linear baud rate */
	private JComboBox  dataBitsInput   = null;						/**< the combo box that show data bits */
	private JComboBox  stopBitsInput   = null;						/**< the combo box that show stop bits */
	private JComboBox  parityInput      = null;						/**< the combo box that show parity */
	private JComboBox  flowCtrlInput   = null;						/**< the combo box that show flow control */
	private JRadioButton  fifoEnableInput  = null;					/**< the radio button that show fifo enable */
	private JRadioButton  fifoDisableInput = null;					/**< the radio button that show fifo disable */
	private ButtonGroup   fifoGroup         = null;					/**< the radio button Group that show fifo */
	private JComboBox  interfaceInput   = null;						/**< the combo box that show serial interface */
	// example
	private JScrollPane readContextScroll = null;					/**< the Text panel that show read context */
	private JScrollPane writeContextScroll = null;					/**< the Text panel that show write context */
	private JTextArea readContextInput = null;						/**< the Text Field that show read context */
	private JTextArea writeContextInput = null;						/**< the Text Field that show write context */
	private JPanel readPanel = null;								/**< the button panel that show read context */
	private JPanel writePanel = null;								/**< the button panel that show write context */
	private JButton sentBut = null;									/**< send button of ezample page */
	private JButton repeatBut = null;								/**< repeat button of ezample page */
	private JButton clearBut = null;								/**< clear button of ezample page */
	private JProgressBar readProgressBar = new JProgressBar(0, 10000);
	private JProgressBar writeProgressBar = new JProgressBar(0, 10000);
	private int modelNameIdx[][] = {
		{0x800071A1, 0x71A2},
		{0x800071A1, 0x71A1},
		{0x800071A3, 0x71A4},
		{0x800071A3, 0x71A3}
	};			/**< apid / hwid table */
	private String modelName[]   = { "MiiNePort E2-H", "MiiNePort E2", "MiiNePort E3-H", "MiiNePort E3"
	                               };								/**< model table table */
	private String baudRateStr[] = null;                       	 	/**< the string array of baud rate and context would set by setModelName function */
	private String dataBitsStr[]
	= {"5", "6", "7", "8"};											/**< the string array of data bits */
	private String stopBitsStr[]
	= {"1", "1.5", "2"};											/**< the string array of stop bits */
	private String parityStr[]
	= {"None", "Odd", "Even", "Mark", "Space"};						/**< the string array of parity bits */
	private String flowCtrlStr[]
	= {"None", "RTS/CTS", "XON/XOFF", "DTR/DSR"};					/**< the string array of flow control */
	private String interfaceStr[]
	= {"RS-232", "RS-485 2Wire"};									/**< the string array of interface */
	// op mode
	private JComboBox  opmodeInput     = null;						/**< the combo box that show opmode */

	private String opmodeStrE2[]
	= {"Disable", "Real COM", "RFC2217", "TCP Server",
	   "TCP Client", "UDP", "Ethernet Modem", "MCSC"
	  };				/**< the string array of op mode bits for MiiNePort E2 */
  private String opmodeStrE3[]
	= {"Disable", "Real COM", "RFC2217", "TCP Server",
	   "TCP Client", "UDP", "Ethernet Modem"
	  };				/**< the string array of op mode bits for MiiNePort E */

	// submit panel
	private JPanel networkPanel = null;
	private JPanel serialPanel  = null;
	private JPanel opmodePanel  = null;
	private JPanel homePanel    = null;
	private FlowLayout flowLayoutNetwork = null;
	private FlowLayout flowLayoutSerial  = null;
	private FlowLayout flowLayoutOpmode  = null;
	private FlowLayout flowLayoutHome    = null;
	// device info
	private String modelNameStr = "Unknown Model";						/**< the model name string */
	private int    modelNo = UNKNOWN_MODEL;						    	/**< the model index */
	private int    portNo  = 1;											/**< the port index */
	private int    localPort = -1;										/**< the local port */
	private String password = "";										/**< the host password */
	private Device device[] = null;										/**< the handler of device */
	// example
	private Socket comConnect = null;									/**< the socket for conncet device */
	private OutputStream  bos  = null;									/**< the output stream for conncet device */
	private InputStream   bis  = null;									/**< the input stream for conncet device */
	private writeThread  wThread = null;								/**< the thread for send pattern */
	private ImageIcon    writeRatePic = null;							/**< the write picture for example */
	private ImageIcon    readRatePic = null;							/**< the read picture for example */
	private drawLabel writeRpsPanel = null;								/**< the label for draw write information on it */
	private drawLabel readRpsPanel  = null;								/**< the label for draw read information on it */

	private Locale locales = null;
	private ResourceBundle resource = null;

	String mainMenuStr = null;										/**< the string : main menu */
	String overviewStr  = null;										/**< the string : overview */
	String basicSettingsStr  = null;									/**< the string : basic settings */
	String networkSettingsStr  = null;								/**< the string : network settings */
	String serialPortSettingsStr  = null;							/**< the string : serial port settings */
	String operationModesStr  = null;									/**< the string : operation modes */
	String exampleStr  = null;											/**< the string : example */
	String tcpServerStr  = null;										/**< the string : tcp server */
	String saveNRestartStr  = null;									/**< the string : save and restart */
	String modeStr  = null;											/**< the string : mode */
	String submitStr  = null;											/**< the string : submit */
	String fontStr = "Verdana";										/**< the font of string : submit */

	loadingFrame startFrame = null;
	// label for draw rps
	private class drawLabel extends JLabel {
		private int rps = 0;								/**< rps counter */
		private Icon ratePic = null;						/**< the picture for rps */
		static public final int	   X_POS  = 45;				/**< start position of x of background picture */
		static public final int	   Y_POS  = 0;				/**< start position of y of background picture */
		static public final int	   BLOCK_VAL = 330;			/**< start position of y of background picture */

		private int rpsPosX[]  = {23, 22, 17, 15, 15, 19, 20, 24, 30, 36, 46, 55, 64, 74, 85, 95, 106, 118, 128, 138, 144, 154, 161, 166, 169, 171, 174, 174, 170, 168, 164};
		private int rpsPosY[]  = {141, 130, 120, 108, 97, 86, 75, 68, 58, 49, 43, 37, 32, 29, 26, 26, 27, 28, 32, 36, 45, 51, 60, 70, 80, 90, 101, 113, 123, 133, 143};

		public drawLabel(Icon rpsImage) {
			super(rpsImage);
			ratePic = rpsImage;
		}
		public void setRps(int rpsValue) {
			rps = rpsValue;
			if (rps <= 0)
				rps = 0;
		}
		public void addRps(int rpsValue) {
			rps += rpsValue;
			if (rps <= 0)
				rps = 0;
		}
		public void paint(Graphics g) {
			int pos = rps / BLOCK_VAL;
			double kb_s = ((double)rps / 1000.0);
			if (pos < 0) pos = 0;
			if (pos >= rpsPosX.length) pos = rpsPosX.length - 1;
			// draw background
			if (ratePic != null)
				ratePic.paintIcon(this, g, X_POS, Y_POS);

			// draw point
			g.setColor(Color.BLUE);
			g.setFont(new Font(fontStr, Font.BOLD, 12));
			g.drawString(kb_s + " kb/s", 60 + X_POS, 85 + Y_POS);
			g.setColor(Color.RED);
			Polygon poly = new Polygon();
			poly.addPoint(91 + X_POS, 148 + Y_POS);
			poly.addPoint(95 + X_POS, 144 + Y_POS);
			poly.addPoint(rpsPosX[pos] + X_POS, rpsPosY[pos] + Y_POS);
			g.fillPolygon(poly);
		}
	}
	// draw thread for example
	private class drawThread extends Thread {
		public drawThread() {
			super("draw");
		}
		public void run() {
			long begin = System.currentTimeMillis();
			long end = 0;
			String pattern = writeContextInput.getText();
			int len = pattern.length();
			do {
				writeRpsPanel.repaint();
				readRpsPanel.repaint();
				end = System.currentTimeMillis();
				if (!sentBut.isEnabled()) {
					// repeat
					begin = System.currentTimeMillis();
				} else {
					// sent
					if ((end - begin) >= 1000) {
						writeRpsPanel.setRps(0);
						readRpsPanel.setRps(0);
						writeProgressBar.setValue(0);
						readProgressBar.setValue(0);
						writeProgressBar.setString("send " + 0 + " bytes");
						readProgressBar.setString("receive " + 0 + " bytes");
						begin = System.currentTimeMillis();
					}
				}
				try {
					Thread.sleep(10);
				} catch (InterruptedException iteo) {
					System.out.println(iteo.toString());
				}
			} while (true);
		}
	}
	// write thread for example
	private class writeThread extends Thread {
		private String pattern = null;
		public writeThread() {
			super("write thread");
		}
		public void run() {
			try {
				long begin = System.currentTimeMillis();
				long end = 0;
				int  counter = 0;
				pattern = writeContextInput.getText();
				bos = comConnect.getOutputStream();
				while (true) {
					bos.write(pattern.getBytes());
					bos.flush();
					counter += pattern.length();
					writeProgressBar.setValue(counter);
					writeProgressBar.setString("send " + counter + " bytes");
					end = System.currentTimeMillis();
					if ((end - begin) >= 1000) {
						writeRpsPanel.setRps(counter);
						counter = 0;
						begin = System.currentTimeMillis();
					}
					Thread.sleep(10);
				}
			} catch (IOException ieo) {
				System.out.println(ieo.toString());
			} catch (InterruptedException iteo) {
				System.out.println(iteo.toString());
			}
		}
	}
	// read thread for example
	private class readThread extends Thread {
		public readThread() {
			super("read thread");
		}
		public void run() {
			try {
				long begin = System.currentTimeMillis();
				long end = 0;
				int  counter = 0;
				bis = comConnect.getInputStream();
				while (true) {
					byte buf[] = new byte[1460];
					int receLen = 0;
					bis.read(buf);
					for (int i = 0; i < buf.length; i++) {
						if (buf[i] == 0x00) break;
						receLen++;
					}

					if (receLen > 0) {
						byte buf_str[] = new byte[receLen];
						for (int i = 0; i < buf_str.length; i++)
							buf_str[i] = buf[i];
						readContextInput.append(new String(buf_str));
					}

					counter += receLen;
					readProgressBar.setValue(counter);
					readProgressBar.setString("receive " + counter + " bytes");
					end = System.currentTimeMillis();
					if ((end - begin) >= 1000) {
						readRpsPanel.setRps(counter);
						counter = 0;
						begin = System.currentTimeMillis();
					}
					Thread.sleep(10);
				}
			} catch (IOException ieo) {
				System.out.println(ieo.toString());
			} catch (InterruptedException iteo) {
				System.out.println(iteo.toString());
			}
		}
	}
	// get resource thread for example
	private class getResThread extends Thread {
		public getResThread() {
			super("get resource thread");
		}
		public void run() {
			// get resource from device
			startFrame = new loadingFrame();
			if (startFrame != null)
				startFrame.setLoadingProgress(25);
			openFolder = createAppletImageIcon("folderop.gif", "Open Node");
			if (startFrame != null)
				startFrame.setLoadingProgress(50);
			closedFolder = createAppletImageIcon("foldercl.gif", "Closed Node");
			if (startFrame != null)
				startFrame.setLoadingProgress(75);
			if (TCPSVREXAMPLE)
				writeRatePic = createAppletImageIcon("rps.JPG", "Write rps");
			if (startFrame != null)
				startFrame.setLoadingProgress(100);
		}
	}
	// start dialog for waiting
	private class loadingFrame extends JFrame {
		private JProgressBar loadingBar = new JProgressBar(0, 100);
		public loadingFrame() {
			super("Loading");
			Container c = getContentPane();
			c.setLayout(new FlowLayout());
			loadingBar.setVisible(true);
			loadingBar.setStringPainted(true);
			loadingBar.setString("Processing...");
			this.setBounds(100, 330, 280, 120);
			c.add(new JLabel("Waiting for loading resource"));
			c.add(loadingBar);
			setVisible(true);
		}

		public void setLoadingProgress(int loading) {
			loadingBar.setValue(loading);
			loadingBar.setString("Loading " + loading + "% ...");
			if (loading >= loadingBar.getMaximum())
				this.setVisible(false);
		}
	}
	/**
	 * \brief	start function for initization.\n
	 * <b>detail description.</b>\n
	 *        it only called by init() \n
	 *        it used to set up GUI. \n
	 *        it would call a thread for loading resource\n
	 */
	public void init() {
		// get resource
		(new getResThread()).start();

		// search device
		try {
			// search time = timeout * retry
			DeviceEnumerator deviceEnumerator = new DeviceEnumerator();
			deviceEnumerator.dsc_SetSearchOpt(timeout, retry);
			device = deviceEnumerator.dsc_EnumSearch(getMyIP());
		} catch (Exception e) {
			device = null;
		}

		if (device == null || device.length <= 0) {
			JOptionPane.showMessageDialog(null, "Device maybe disconnect", "Results", JOptionPane.PLAIN_MESSAGE);
			return;
		}
		// set model name
		setModelName();

		// is support multi-language
		if (MULTILANGUAGE) {
			try {
				locales = new Locale("", "");
			} catch (Exception e) {
				System.out.println("[ERROR] : " + e.toString());
				locales = Locale.getDefault();
			}

			ResourceBundle resource = ResourceBundle.getBundle("messages", locales);

			// read string from properties file
			mainMenuStr = resource.getString("ezpage.mainmenu");
			overviewStr = resource.getString("ezpage.overview");
			basicSettingsStr  = resource.getString("ezpage.basicsettings");
			networkSettingsStr  = resource.getString("ezpage.networksettings");
			serialPortSettingsStr  = resource.getString("ezpage.serialportsettings");
			operationModesStr  = resource.getString("ezpage.operationmodes");
			exampleStr  = resource.getString("ezpage.example");
			tcpServerStr  = resource.getString("ezpage.tcpserver");
			saveNRestartStr  = resource.getString("ezpage.savenrestart");
			modeStr  = resource.getString("ezpage.mode");
			submitStr  = resource.getString("ezpage.submit");
		} else {
			mainMenuStr = "Main Menu";
			overviewStr = "Overview";
			basicSettingsStr  = "Basic Settings";
			networkSettingsStr  = "Network Settings";
			serialPortSettingsStr  = "Serial Port Settings";
			operationModesStr  = "Operation Modes";
			exampleStr  = "Example";
			tcpServerStr  = "TCP Server";
			saveNRestartStr  = "Save and Restart";
			modeStr  = "Mode";
			submitStr  = "Submit";
		}
		// page layout of main menu
		mainPage = getContentPane();
		gbLayout = new GridBagLayout();
		containPage = new GridBagConstraints();
		containPage.fill = GridBagConstraints.BOTH;
		containPage.insets = new Insets(3, 3, 3, 3);
		rightPage = new JPanel();
		rightPage.setLayout(gbLayout);
		rightPage.setBackground(Color.WHITE);

		networkPanel = new JPanel();
		serialPanel  = new JPanel();
		opmodePanel  = new JPanel();
		homePanel    = new JPanel();
		flowLayoutNetwork = new FlowLayout();
		flowLayoutSerial  = new FlowLayout();
		flowLayoutOpmode  = new FlowLayout();
		flowLayoutHome    = new FlowLayout();
		flowLayoutNetwork.setAlignment(FlowLayout.LEFT);
		flowLayoutSerial.setAlignment(FlowLayout.LEFT);
		flowLayoutOpmode.setAlignment(FlowLayout.LEFT);
		flowLayoutHome.setAlignment(FlowLayout.LEFT);
		networkPanel.setBackground(Color.WHITE);
		serialPanel.setBackground(Color.WHITE);
		opmodePanel.setBackground(Color.WHITE);
		homePanel.setBackground(Color.WHITE);
		networkPanel.setLayout(flowLayoutNetwork);
		serialPanel.setLayout(flowLayoutSerial);
		opmodePanel.setLayout(flowLayoutOpmode);
		homePanel.setLayout(flowLayoutHome);

		// init left tree view
//		openFolder = createAppletImageIcon("folderop.gif", "Open Node");
//		closedFolder = createAppletImageIcon("foldercl.gif", "Closed Node");
		do {
			try {
				Thread.sleep(10);
			} catch (InterruptedException iteo) {
				System.out.println(iteo.toString());
			}
		} while (openFolder == null || closedFolder == null);
		TreeNode root = makeTree();
		model = new DefaultTreeModel(root);
		tree = new JTree(model);
		tree.setEditable(false);
		MouseListener ml = new MouseAdapter() {
			public void mousePressed(MouseEvent e) {
				TreePath selPath = tree.getPathForLocation(e.getX(), e.getY());
				int selRow = tree.getRowForLocation(e.getX(), e.getY());
				if (selRow != -1) {
					clickTreeView(selPath);
				}
			}
		};
		DefaultTreeCellRenderer renderer = new DefaultTreeCellRenderer();
		renderer.setBackgroundNonSelectionColor(null);
		renderer.setTextNonSelectionColor(new Color(0x00, 0x00, 0xff));
		renderer.setTextSelectionColor(new Color(0x00, 0x00, 0xff));
		renderer.setLeafIcon(closedFolder);
		renderer.setClosedIcon(closedFolder);
		renderer.setOpenIcon(openFolder);
		tree.setCellRenderer(renderer);
		tree.addMouseListener(ml);
		tree.setFont(new Font(fontStr, Font.PLAIN, 10));
		tree.setOpaque(true);
		tree.setForeground(new Color(0x00, 0x00, 0xff));
		tree.setBackground(new Color(0xcc, 0xf4, 0xde));
		tree.setPreferredSize(new Dimension(200, 800));

		// page layout of network settings
		comboHandler = new ComboHandler();
		deviceNameInput = new JTextField(39);
		ipCfgInput = new JComboBox(ipCfgStr);
		ipCfgInput.addItemListener(comboHandler);
		ipInput = new JTextField(15);
		netmaskInput = new JTextField(15);
		gatewayInput = new JTextField(15);
		dns1Input = new JTextField(15);
		dns2Input = new JTextField(15);

		// page layout of serial port settings
		portAliasInput   = new JTextField(15);
		linearBaudInput  = new JTextField(6);
		baudRateInput    = new JComboBox(baudRateStr);
		dataBitsInput    = new JComboBox(dataBitsStr);
		stopBitsInput    = new JComboBox(stopBitsStr);
		parityInput       = new JComboBox(parityStr);
		flowCtrlInput    = new JComboBox(flowCtrlStr);
		interfaceInput    = new JComboBox(interfaceStr);
		fifoEnableInput  = new JRadioButton("Enable", true);
		fifoDisableInput = new JRadioButton("Disable", false);
		fifoGroup         = new ButtonGroup();
		fifoGroup.add(fifoEnableInput);
		fifoGroup.add(fifoDisableInput);
		baudRateInput.addItemListener(comboHandler);

		fifoEnableInput.setBackground(Color.WHITE);
		fifoDisableInput.setBackground(Color.WHITE);

		if (TCPSVREXAMPLE) {
			// page layout of example
			readPanel = new JPanel();
			writePanel =  new JPanel();
			readContextInput  = new JTextArea("", 8, 20);
			writeContextInput = new JTextArea("", 8, 20);
			readContextScroll  = new JScrollPane(readContextInput);
			writeContextScroll = new JScrollPane(writeContextInput);

//			writeRatePic  = createAppletImageIcon("rps.JPG", "Write rps");

			do {
				try {
					Thread.sleep(10);
				} catch (InterruptedException iteo) {
					System.out.println(iteo.toString());
				}
			} while (writeRatePic == null);

			readRatePic = writeRatePic;

			sentBut = new JButton("Sent");
			repeatBut = new JButton("Repeat");
			clearBut = new JButton("Clear");
			writeRpsPanel = new drawLabel(writeRatePic);
			readRpsPanel  = new drawLabel(readRatePic);
			readContextInput.setEnabled(false);
			readContextInput.setBackground(new Color(0xcc, 0xf4, 0xde));
			writeContextInput.setBackground(new Color(0xcc, 0xf4, 0xde));
			writePanel.setBackground(Color.WHITE);
			readPanel.setBackground(Color.WHITE);
			writePanel.add(sentBut);
			writePanel.add(repeatBut);
			readPanel.add(clearBut);
		}

		// save and restart

		// add action listener
		networkSubmit = new JButton(submitStr);
		serialSubmit = new JButton(submitStr);
		opmodeSubmit = new JButton(submitStr);
		back = new JButton("Back");
		restart = new JButton("Restart");
		home = new JButton("Home");
		handler = new ButtonHandler();
		networkSubmit.addActionListener(handler);
		serialSubmit.addActionListener(handler);
		opmodeSubmit.addActionListener(handler);
		back.addActionListener(handler);
		restart.addActionListener(handler);
		home.addActionListener(handler);
		if (TCPSVREXAMPLE) {
			sentBut.addActionListener(handler);
			repeatBut.addActionListener(handler);
			clearBut.addActionListener(handler);
		}

		// set button panel
		networkPanel.add(networkSubmit);
		serialPanel.add(serialSubmit);
		opmodePanel.add(opmodeSubmit);

		// get device information
		getDeviceInfo();

		// overview page
		containPage.anchor = GridBagConstraints.NORTHWEST;
		overviewRefresh();

		// set main page
		mainPage.add(tree, BorderLayout.WEST);
		mainPage.add(rightPage, BorderLayout.CENTER);
	}
	/**
	 * \brief	end function for close.\n
	 * <b>detail description.</b>\n
	 *        it would close tcp client connection when user close brower\n
	 */
	public void destory() {
		closeTcpServer();
	}
	/**
	 * \brief	button handler.\n
	 * <b>detail description.</b>\n
	 *        user click buttn would call this handler function \n
	 */
	private class ButtonHandler implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			if (e.getSource() == networkSubmit)
				networkSubmit();
			else if (e.getSource() == serialSubmit)
				serialSubmit();
			else if (e.getSource() == opmodeSubmit)
				opmodeSubmit();
			else if (e.getSource() == back)
				doBack();
			else if (e.getSource() == restart)
				doSaveRestart();
			else if (e.getSource() == home) {
				getDeviceInfo();
				overviewRefresh();
			} else if (e.getSource() == sentBut) {
				sendDataTcpServer();
			} else if (e.getSource() == repeatBut) {
				sendPatternTcpServer();
			} else if (e.getSource() == clearBut) {
				writeContextInput.setText("");
				readContextInput.setText("");
			}
		}
	}
	/**
	 * \brief	combo box handler.\n
	 * <b>detail description.</b>\n
	 *        user change combo box item would call this handler function \n
	 */
	private class ComboHandler implements ItemListener {
		public void itemStateChanged(ItemEvent e) {
			if (e.getSource() == ipCfgInput)
				setNetworkState();
			else if (e.getSource() == baudRateInput)
				setSerialState();
		}
	}
	/**
	 * \brief	update network state.\n
	 * <b>detail description.</b>\n
	 *        update network state when user change ip configuration combo box items \n
	 */
	private void setNetworkState() {
		if (ipCfgInput.getSelectedIndex() == 0) {
			ipInput.setEnabled(true);
			netmaskInput.setEnabled(true);
			gatewayInput.setEnabled(true);
			dns1Input.setEnabled(true);
			dns2Input.setEnabled(true);
		} else {
			ipInput.setEnabled(false);
			netmaskInput.setEnabled(false);
			gatewayInput.setEnabled(false);
			dns1Input.setEnabled(false);
			dns2Input.setEnabled(false);
		}
	}
	/**
	 * \brief	update serial state.\n
	 * <b>detail description.</b>\n
	 *        update serial state when user change baud rate combo box items \n
	 */
	private void setSerialState() {
		if (baudRateInput.getSelectedIndex() == baudRateStr.length - 1)
			linearBaudInput.setEnabled(true);
		else
			linearBaudInput.setEnabled(false);
	}
	/**
	 * \brief	get configuration from device.\n
	 * <b>detail description.</b>\n
	 *        get configuration from device and set component's data \n
	 */
	private void getDeviceInfo() {
		int    up_time_num = device[0].MX_INT_FAILED;
		int    ip_cfg_num  = device[0].MX_INT_FAILED;
		int    interface_num = device[0].MX_INT_FAILED;
		int    op_mode_num = device[0].MX_FAILED;
		byte   fifo_num    = device[0].MX_FAILED;
		String device_name_str = "";
		String port_name_str   = "";
		java.net.InetAddress host_ip    = null;
		java.net.InetAddress netmask_ip = null;
		java.net.InetAddress gateway_ip = null;
		java.net.InetAddress dns_ip[]   = null;
		serialNoInfo    = null;
		//deviceNameInfo  = null;
		firmwareVerInfo = null;
		ethIpInfo  = null;
		ethMacInfo = null;
		upTimeInfo = null;
		serialComStatusInfo  = null;

		try {
			// get device information
			device[0].dsc_Attach();
			Device.KERNELINFO  kernelInfo = device[0].dsc_GetKernelInfo();
			device_name_str = device[0].dsc_GetName();
			host_ip = device[0].getSearchInfo().ip1;
			ip_cfg_num = device[0].dsc_GetIPConfig(0xff);

			netmask_ip  = device[0].dsc_GetNetmask(0xff);
			gateway_ip  = device[0].dsc_GetGateway(0xff);
			dns_ip      = device[0].dsc_GetDNS(0xff);

			op_mode_num = device[0].dsc_GetOpModeEx(portNo, 0);

			Device.SERPARMITEM  serial_info = device[0].dsc_GetSerialIOCtl(portNo, device[0].MODE_CONFIG);
			fifo_num = device[0].dsc_GetSerialFIFO(portNo, device[0].MODE_CONFIG);

			interface_num = device[0].dsc_GetSerialInterface(portNo, device[0].MODE_CONFIG);

			port_name_str = device[0].dsc_GetSerialPortName(portNo);
			Device.SCMD_INFO scm_info = device[0].dsc_GetCmdState();

			if (deviceNameInfo == null)
				deviceNameInfo  = new JLabel(device_name_str);

			// overview
			if (kernelInfo != null) {
				// input password
				if ((kernelInfo.flag & 0x01) == 1) {
					while (true) {
						boolean isBreak = false;
						JPasswordField pwd = new JPasswordField();
						Object[] message = {"Input password please", pwd};
						int res = JOptionPane.showConfirmDialog(null, message, "Input password please",
						                                        JOptionPane.OK_CANCEL_OPTION, JOptionPane.QUESTION_MESSAGE);
						password = new String(pwd.getPassword());
						//password = JOptionPane.showInputDialog("Input password please");
						try {
							device[0].dsc_Login(password);
							isBreak = true;
						} catch (Exception e) {
							JOptionPane.showMessageDialog(null, "Login failed", "Error Message", JOptionPane.ERROR_MESSAGE);
						}
						if (isBreak)
							break;
					}

					device[0].dsc_Logout();
				}

				serialNoInfo  = new JLabel("" + kernelInfo.serial_no);
				int major_num = (int)kernelInfo.firm_version;
				int minor_num = (int)kernelInfo.bios_version;

				firmwareVerInfo  = new JLabel(major_num + "." + minor_num);
			} else {
				serialNoInfo  = new JLabel("0");
				firmwareVerInfo  = new JLabel("0.0");
			}

			if (host_ip != null) {
				ethIpInfo  = new JLabel(checkIpFormat(host_ip.toString()));
				ipInput.setText(checkIpFormat(host_ip.toString()));
			} else {
				ethIpInfo  = new JLabel("");
				ipInput.setText("");
			}

			ethMacInfo  = new JLabel(Integer.toHexString((device[0].getSearchInfo().mac1)[0] & 0xff) + ":" +
			                         Integer.toHexString((device[0].getSearchInfo().mac1)[1] & 0xff) + ":" +
			                         Integer.toHexString((device[0].getSearchInfo().mac1)[2] & 0xff) + ":" +
			                         Integer.toHexString((device[0].getSearchInfo().mac1)[3] & 0xff) + ":" +
			                         Integer.toHexString((device[0].getSearchInfo().mac1)[4] & 0xff) + ":" +
			                         Integer.toHexString((device[0].getSearchInfo().mac1)[5] & 0xff));

			up_time_num = device[0].dsc_GetSysUpTime();
			if (up_time_num != device[0].MX_INT_FAILED) {
				upTimeInfo = new JLabel(up_time_num / (60 * 60 * 24) + " days " + up_time_num / (60 * 60) + "h:"
				                        + up_time_num / 60 + "m:" + up_time_num % 60 + "s");
			} else {
				upTimeInfo = new JLabel("0 days 00h:00m:00s");
			}

			if (scm_info != null) {
				if (scm_info.scm[4] == 0x00)
					serialComStatusInfo  = new JLabel("Data Mode");
				else if (scm_info.scm[4] == 0x01)
					serialComStatusInfo  = new JLabel("Serial Command Mode");
				else if (scm_info.scm[4] == 0x02)
					serialComStatusInfo  = new JLabel("MCSC enabled");
				else if (scm_info.scm[0] == 0x82)
					serialComStatusInfo  = new JLabel("booting up");
				else
					serialComStatusInfo  = new JLabel("");
			} else {
				serialComStatusInfo  = new JLabel("");
			}
			// network
			deviceNameInput.setText(device_name_str);
			if (ip_cfg_num == device[0].MX_INT_FAILED)
				ipCfgInput.setSelectedIndex(0);
			else
				ipCfgInput.setSelectedIndex(ip_cfg_num);
			setNetworkState();
			if (netmask_ip != null)
				netmaskInput.setText(checkIpFormat(netmask_ip.toString()));
			else
				netmaskInput.setText("");
			if (gateway_ip != null)
				gatewayInput.setText(checkIpFormat(gateway_ip.toString()));
			else
				gatewayInput.setText("");
			if (dns1Input != null) {
				dns1Input.setText(checkIpFormat(dns_ip[0].toString()));
				dns2Input.setText(checkIpFormat(dns_ip[1].toString()));
			} else {
				dns1Input.setText("");
				dns2Input.setText("");
			}
			// serial
			portAliasInput.setText(port_name_str);
			if (serial_info != null) {
				boolean set = false;
				byte parity = 0x00;
				for (int i = 0; i < baudRateStr.length - 1; i++) {
					if (Integer.parseInt(baudRateStr[i]) == serial_info.baud) {
						baudRateInput.setSelectedIndex(i);
						set = true;
					}
				}
				if (!set) {
					baudRateInput.setSelectedIndex(baudRateStr.length - 1);
					linearBaudInput.setText(serial_info.baud + "");
				} else
					linearBaudInput.setText("");
				// data bits
				if ((serial_info.mode & 0x03) == 0x00)
					dataBitsInput.setSelectedIndex(0);
				else if ((serial_info.mode & 0x03) == 0x01)
					dataBitsInput.setSelectedIndex(1);
				else if ((serial_info.mode & 0x03) == 0x02)
					dataBitsInput.setSelectedIndex(2);
				else
					dataBitsInput.setSelectedIndex(3);
				// stop bits
				if (((serial_info.mode & 0x03) == 0x00) && ((serial_info.mode & 0x04) == 0x04))
					stopBitsInput.setSelectedIndex(1);
				else if ((serial_info.mode & 0x04) == 0x04)
					stopBitsInput.setSelectedIndex(2);
				else
					stopBitsInput.setSelectedIndex(0);
				// parity
				parity = (byte)(serial_info.mode & 0x38);

				if (parity == device[0].MX_PARITY_ODD)
					parityInput.setSelectedIndex(1);
				else if (parity == device[0].MX_PARITY_EVEN)
					parityInput.setSelectedIndex(2);
				else if (parity == device[0].MX_PARITY_MARK)
					parityInput.setSelectedIndex(3);
				else if (parity == device[0].MX_PARITY_SPACE)
					parityInput.setSelectedIndex(4);
				else
					parityInput.setSelectedIndex(0);
				flowCtrlInput.setSelectedIndex(serial_info.flowctrl);
			} else {
				baudRateInput.setSelectedIndex(16);
				dataBitsInput.setSelectedIndex(3);
				stopBitsInput.setSelectedIndex(0);
				parityInput.setSelectedIndex(0);
				flowCtrlInput.setSelectedIndex(1);

			}
			if (fifo_num == 0x01) {
				fifoEnableInput.setSelected(true);
				fifoDisableInput.setSelected(false);
			} else {
				fifoEnableInput.setSelected(false);
				fifoDisableInput.setSelected(true);
			}

			switch (interface_num) {
				case 0x00:
					interfaceInput.setSelectedIndex(0);
					break;
				case 0x02:
					interfaceInput.setSelectedIndex(1);
					break;
				default:
					interfaceInput.setSelectedIndex(0);
			}
			setSerialState();
			// op mode
			switch (op_mode_num) {
				case 0x02:
					opmodeInput.setSelectedIndex(1);
					break;
				case 0x14:
					opmodeInput.setSelectedIndex(2);
					break;
				case 0x0A:
					opmodeInput.setSelectedIndex(3);
					break;
				case 0x0D:
					opmodeInput.setSelectedIndex(4);
					break;
				case 0x0E:
					opmodeInput.setSelectedIndex(5);
					break;
				case 0x0C:
					opmodeInput.setSelectedIndex(6);
					break;
				case 0x40:
					opmodeInput.setSelectedIndex(7);
					break;
				case 0x07:
					opmodeInput.setSelectedIndex(0);
					break;
				default:
					opmodeInput.setSelectedIndex(0);
			}

			device[0].dsc_Detach();

		} catch (Exception e) {
		}
	}
	/**
	 * \brief	refresh overview page.\n
	 * <b>detail description.</b>\n
	 *        refresh overview page \n
	 */
	private void overviewRefresh() {
		rightPage.removeAll();
		// init data
		JLabel empty = new JLabel("");
		JLabel title = new JLabel(" Welcome to " + modelNameStr + " web console");
		JLabel modelName = new JLabel(" Model name");
		JLabel serial_no  = new JLabel(" Serial No.");
		JLabel device_name  = new JLabel(" Device name");
		JLabel firmware_ver = new JLabel(" Firmware version");
		JLabel eth_ip  = new JLabel(" Ethernet IP address");
		JLabel eth_mac = new JLabel(" Ethernet MAC address");
		JLabel up_time = new JLabel(" Up time");
		JLabel serial_com_status  = new JLabel(" Serial communication status");
		JLabel modelName_info = new JLabel(modelNameStr);

		// set label type
		setLabelType(title, 0);
		setLabelType(modelName, 1);
		setLabelType(serial_no, 1);
		setLabelType(device_name, 1);
		setLabelType(firmware_ver, 1);
		setLabelType(eth_ip, 1);
		setLabelType(eth_mac, 1);
		setLabelType(up_time, 1);
		setLabelType(serial_com_status, 1);
		setLabelType(modelName_info, 2);
		setLabelType(serialNoInfo, 2);
		setLabelType(deviceNameInfo, 2);
		setLabelType(firmwareVerInfo, 2);
		setLabelType(ethIpInfo, 2);
		setLabelType(ethMacInfo, 2);
		setLabelType(upTimeInfo, 2);
		setLabelType(serialComStatusInfo, 2);
		setLabelType(modelName, 4);
		setLabelType(modelName_info, 4);
		setLabelType(serial_no, 5);
		setLabelType(serialNoInfo, 5);
		setLabelType(device_name, 4);
		setLabelType(deviceNameInfo, 4);
		setLabelType(firmware_ver, 5);
		setLabelType(firmwareVerInfo, 5);
		setLabelType(eth_ip, 4);
		setLabelType(ethIpInfo, 4);
		setLabelType(eth_mac, 5);
		setLabelType(ethMacInfo, 5);
		setLabelType(up_time, 4);
		setLabelType(upTimeInfo, 4);
		setLabelType(serial_com_status, 5);
		setLabelType(serialComStatusInfo, 5);

		// add component
		containPage.weightx = 1;
		containPage.weighty = 0;
		addComponent(title, 0, 0, 3, 1);
		addComponent(modelName, 1, 0, 1, 1);
		addComponent(modelName_info, 1, 1, 1, 1);
		addComponent(serial_no, 2, 0, 1, 1);
		addComponent(serialNoInfo, 2, 1, 1, 1);
		addComponent(device_name, 3, 0, 1, 1);
		addComponent(deviceNameInfo, 3, 1, 1, 1);
		addComponent(firmware_ver, 4, 0, 1, 1);
		addComponent(firmwareVerInfo, 4, 1, 1, 1);
		addComponent(eth_ip, 5, 0, 1, 1);
		addComponent(ethIpInfo, 5, 1, 1, 1);
		addComponent(eth_mac, 6, 0, 1, 1);
		addComponent(ethMacInfo, 6, 1, 1, 1);
		addComponent(up_time, 7, 0, 1, 1);
		addComponent(upTimeInfo, 7, 1, 1, 1);
		addComponent(serial_com_status, 8, 0, 1, 1);
		addComponent(serialComStatusInfo, 8, 1, 1, 1);

		containPage.weightx = 1;
		containPage.weighty = 1;
		addComponent(empty, 9, 0, 2, 1);

		backIdx = OVERVIEW_PAGE;
		rightPage.validate();
		rightPage.updateUI();
	}
	/**
	 * \brief	refresh network page.\n
	 * <b>detail description.</b>\n
	 *        refresh network page \n
	 */
	private void networkRefresh() {
		rightPage.removeAll();
		// init data
		JLabel empty = new JLabel("");
		JLabel title = new JLabel(" Network Settings");
		JLabel device_name  = new JLabel(" Device name");
		JLabel ip_cfg   = new JLabel(" IP configuration");
		JLabel ip       = new JLabel(" IP address");
		JLabel netmask  = new JLabel(" Netmask");
		JLabel gateway  = new JLabel(" Gateway");
		JLabel dns1     = new JLabel(" DNS server 1");
		JLabel dns2     = new JLabel(" DNS server 2");
		JLabel empty1 = new JLabel("");
		JLabel empty2 = new JLabel("");
		JLabel empty3 = new JLabel("");
		JLabel empty4 = new JLabel("");
		JLabel empty5 = new JLabel("");
		JLabel empty6 = new JLabel("");
		JLabel empty7 = new JLabel("");
		JLabel empty8 = new JLabel("");

		// set label type
		setLabelType(title, 0);
		setLabelType(device_name, 0);
		setLabelType(ip_cfg, 0);
		setLabelType(ip, 0);
		setLabelType(netmask, 0);
		setLabelType(gateway, 0);
		setLabelType(dns1, 0);
		setLabelType(dns2, 0);

		// add component
		containPage.weightx = 1;
		containPage.weighty = 0;
		addComponent(title, 0, 0, 2, 1);
		addComponent(device_name, 1, 0, 1, 1);
		addComponent(deviceNameInput, 1, 1, 1, 1);
		addComponent(empty1, 1, 2, 1, 1);
		addComponent(ip_cfg, 2, 0, 1, 1);
		addComponent(ipCfgInput, 2, 1, 1, 1);
		addComponent(empty2, 2, 2, 1, 1);
		addComponent(ip, 3, 0, 1, 1);
		addComponent(ipInput, 3, 1, 1, 1);
		addComponent(empty3, 3, 2, 1, 1);
		addComponent(netmask, 4, 0, 1, 1);
		addComponent(netmaskInput, 4, 1, 1, 1);
		addComponent(empty4, 4, 2, 1, 1);
		addComponent(gateway, 5, 0, 1, 1);
		addComponent(gatewayInput, 5, 1, 1, 1);
		addComponent(empty5, 5, 2, 1, 1);
		addComponent(dns1, 6, 0, 1, 1);
		addComponent(dns1Input, 6, 1, 1, 1);
		addComponent(empty6, 6, 2, 1, 1);
		addComponent(dns2, 7, 0, 1, 1);
		addComponent(dns2Input, 7, 1, 1, 1);
		addComponent(empty7, 7, 2, 1, 1);
		addComponent(networkPanel, 8, 0, 1, 1);
		addComponent(empty8, 8, 1, 2, 1);

		containPage.weightx = 1;
		containPage.weighty = 1;
		addComponent(empty, 9, 0, 3, 1);

		backIdx = NETWORK_PAGE;
		rightPage.validate();
		rightPage.updateUI();
	}
	/**
	 * \brief	submit network page.\n
	 * <b>detail description.</b>\n
	 *        submit network page and config device's settings about network \n
	 */
	private void networkSubmit() {
		String device_name_str = deviceNameInput.getText();
		int    ip_cfg_int      = ipCfgInput.getSelectedIndex();
		String ip_str          = ipInput.getText();
		String netmask_str     = netmaskInput.getText();
		String gateway_str     = gatewayInput.getText();
		String dns1_str        = dns1Input.getText();
		String dns2_str        = dns2Input.getText();

		// check value
		if (device_name_str.length() > 39) {
			JOptionPane.showMessageDialog(null, "Device name is over max length", "Error Message", JOptionPane.ERROR_MESSAGE);
			return;
		}

		if (ip_cfg_int == 0) {
			if (ip_str.length() > 15) {
				JOptionPane.showMessageDialog(null, "IP address is over max length", "Error Message", JOptionPane.ERROR_MESSAGE);
				return;
			}

			if (netmask_str.length() > 15) {
				JOptionPane.showMessageDialog(null, "Netmask is over max length", "Error Message", JOptionPane.ERROR_MESSAGE);
				return;
			}

			if (gateway_str.length() > 15) {
				JOptionPane.showMessageDialog(null, "Gateway is over max length", "Error Message", JOptionPane.ERROR_MESSAGE);
				return;
			}

			if (dns1_str.length() > 15) {
				JOptionPane.showMessageDialog(null, "DNS1 is over max length", "Error Message", JOptionPane.ERROR_MESSAGE);
				return;
			}

			if (dns2_str.length() > 15) {
				JOptionPane.showMessageDialog(null, "DNS2 is over max length", "Error Message", JOptionPane.ERROR_MESSAGE);
				return;
			}

			if (gateway_str.equals(""))
				gateway_str = "0.0.0.0";
			if (dns1_str.equals(""))
				dns1_str = "0.0.0.0";
			if (dns2_str.equals(""))
				dns2_str = "0.0.0.0";

			if (!isIp(ip_str)) {
				JOptionPane.showMessageDialog(null, "IP address format is invalid", "Error Message", JOptionPane.ERROR_MESSAGE);
				return;
			}

			if (!isIp(netmask_str)) {
				JOptionPane.showMessageDialog(null, "Netmask format is invalid", "Error Message", JOptionPane.ERROR_MESSAGE);
				return;
			}

			if (!isIp(gateway_str)) {
				JOptionPane.showMessageDialog(null, "Gateway format is invalid", "Error Message", JOptionPane.ERROR_MESSAGE);
				return;
			}

			if (!isIp(dns1_str)) {
				JOptionPane.showMessageDialog(null, "DNS1 format is invalid", "Error Message", JOptionPane.ERROR_MESSAGE);
				return;
			}

			if (!isIp(dns2_str)) {
				JOptionPane.showMessageDialog(null, "DNS2 format is invalid", "Error Message", JOptionPane.ERROR_MESSAGE);
				return;
			}
		}

		try {
			// set device configuration
			device[0].dsc_Attach();
			device[0].dsc_Login(password);
			device[0].dsc_SetName(device_name_str);
			if (ip_cfg_int == 0) {
				java.net.InetAddress new_ip = java.net.InetAddress.getByName(ip_str);
				java.net.InetAddress new_netmask = java.net.InetAddress.getByName(netmask_str);
				java.net.InetAddress new_gateway = java.net.InetAddress.getByName(gateway_str);
				java.net.InetAddress new_dns1 = java.net.InetAddress.getByName(dns1_str);
				java.net.InetAddress new_dns2 = java.net.InetAddress.getByName(dns2_str);

				device[0].dsc_SetIP(0xff, new_ip);
				device[0].dsc_SetNetmask(0xff, new_netmask);
				device[0].dsc_SetGateway(0xff, new_gateway);
				device[0].dsc_SetDNS(0xff, new_dns1, new_dns2);
			}
			device[0].dsc_SetIPConfig(0xff, ip_cfg_int);
			device[0].dsc_SaveAndRestart();
			device[0].dsc_Logout();
			device[0].dsc_Detach();
			dsc_SaveAndRestartRefresh();

		} catch (Exception e) {
			JOptionPane.showMessageDialog(null, "Set failed", "Error Message", JOptionPane.ERROR_MESSAGE);
			return;
		}
	}
	/**
	 * \brief	refresh serial page.\n
	 * <b>detail description.</b>\n
	 *        refresh serial page \n
	 */
	private void serialRefresh() {
		rightPage.removeAll();
		// init data
		JLabel empty = new JLabel("");
		JLabel title = new JLabel(" Communication Parameters");
		JLabel port_info  = new JLabel(" Port " + portNo);
		JLabel port_alias = new JLabel(" Port alias");
		JLabel title2     = new JLabel(" Serial Parameters");
		JLabel baudRate  = new JLabel(" Baud rate");
		JLabel dataBits  = new JLabel(" Data bits");
		JLabel stopBits  = new JLabel(" Stop bits");
		JLabel parity     = new JLabel(" Parity");
		JLabel flowCtrl  = new JLabel(" Flow control");
		JLabel fifo       = new JLabel(" FIFO");
		JLabel inf        = new JLabel(" Interface");

		// set label type
		setLabelType(title, 0);
		setLabelType(port_info, 6);
		setLabelType(title2, 6);
		setLabelType(port_alias, 0);
		setLabelType(baudRate, 0);
		setLabelType(dataBits, 0);
		setLabelType(stopBits, 0);
		setLabelType(parity, 0);
		setLabelType(flowCtrl, 0);
		setLabelType(fifo, 0);
		setLabelType(inf, 0);

		// add component
		containPage.weightx = 1;
		containPage.weighty = 0;
		addComponent(title, 0, 0, 1, 1);
		addComponent(port_info, 1, 0, 4, 1);
		addComponent(port_alias, 2, 0, 1, 1);
		addComponent(portAliasInput, 2, 1, 3, 1);
		addComponent(title2, 3, 0, 4, 1);
		addComponent(baudRate, 4, 0, 1, 1);
		addComponent(baudRateInput, 4, 1, 2, 1);
		addComponent(linearBaudInput, 4, 3, 1, 1);
		addComponent(dataBits, 5, 0, 1, 1);
		addComponent(dataBitsInput, 5, 1, 2, 1);
		addComponent(stopBits, 6, 0, 1, 1);
		addComponent(stopBitsInput, 6, 1, 2, 1);
		addComponent(parity, 7, 0, 1, 1);
		addComponent(parityInput, 7, 1, 2, 1);
		addComponent(flowCtrl, 8, 0, 1, 1);
		addComponent(flowCtrlInput, 8, 1, 2, 1);
		addComponent(fifo, 9, 0, 1, 1);
		addComponent(fifoEnableInput, 9, 1, 1, 1);
		addComponent(fifoDisableInput, 9, 2, 1, 1);

		// MiiNePort E2
		addComponent(inf, 10, 0, 1, 1);
		addComponent(interfaceInput, 10, 1, 1, 1);
		addComponent(serialPanel, 11, 0, 1, 1);


		containPage.weightx = 1;
		containPage.weighty = 1;

		// MiiNePort E2
		addComponent(empty, 12, 0, 4, 1);

		backIdx = SERIAL_PAGE;
		rightPage.validate();
		rightPage.updateUI();
	}
	/**
	 * \brief	submit serial page.\n
	 * <b>detail description.</b>\n
	 *        submit serial page and config device's settings about serial \n
	 */
	private void serialSubmit() {
		int baud_idx = 0;
		int fifo = 0x01;
		String port_name = "";
		Device.ASYNCIOCTL  serial_io_info = device[0].new ASYNCIOCTL();
		serial_io_info.baud = 16;
		serial_io_info.mode = 0x03;
		serial_io_info.flowctrl = 1;

		port_name = portAliasInput.getText();

		baud_idx = baudRateInput.getSelectedIndex();
		try {
			if (baud_idx == baudRateStr.length - 1)
				serial_io_info.baud = Integer.parseInt(linearBaudInput.getText());
			else
				serial_io_info.baud = Integer.parseInt(baudRateStr[baud_idx]);
		} catch (Exception e) {
			JOptionPane.showMessageDialog(null, "Baud rate is format error", "Error Message", JOptionPane.ERROR_MESSAGE);
			return;
		}
		serial_io_info.mode = (byte)(dataBitsInput.getSelectedIndex());
		if (stopBitsInput.getSelectedIndex() == 0)
			serial_io_info.mode |= 0x00;
		else
			serial_io_info.mode |= 0x04;

		if (parityInput.getSelectedIndex() == 1)
			serial_io_info.mode |= device[0].MX_PARITY_ODD;
		else if (parityInput.getSelectedIndex() == 2)
			serial_io_info.mode |= device[0].MX_PARITY_EVEN;
		else if (parityInput.getSelectedIndex() == 3)
			serial_io_info.mode |= device[0].MX_PARITY_MARK;
		else if (parityInput.getSelectedIndex() == 4)
			serial_io_info.mode |= device[0].MX_PARITY_SPACE;
		else
			serial_io_info.mode |= device[0].MX_PARITY_NONE;

		serial_io_info.flowctrl = (byte)flowCtrlInput.getSelectedIndex();

		if (fifoEnableInput.isSelected())
			fifo = 0x01;
		else
			fifo = 0x00;

		// check value
		if (port_name.length() > 15) {
			JOptionPane.showMessageDialog(null, "Port name is over max length", "Error Message", JOptionPane.ERROR_MESSAGE);
			return;
		}

		if (baudRateInput.getSelectedIndex() == baudRateStr.length - 1) {
			if (serial_io_info.baud > Integer.parseInt(baudRateStr[baudRateStr.length-2]) ||
			        serial_io_info.baud < Integer.parseInt(baudRateStr[0])) {
				JOptionPane.showMessageDialog(null, "Baud rate is over range", "Error Message", JOptionPane.ERROR_MESSAGE);
				return;
			}
		}

		// set device configuration
		try {
			device[0].dsc_Attach();
			device[0].dsc_Login(password);

			device[0].dsc_SetSerialPortName(portNo, port_name);
			device[0].dsc_SetAsyncIoctl(portNo, serial_io_info);
			device[0].dsc_SetSerialFIFO(portNo, fifo);

			int interface_num = device[0].MX_INF_RS232;
			switch (interfaceInput.getSelectedIndex()) {
				case 0:
					interface_num = device[0].MX_INF_RS232;
					break;
				case 1:
					interface_num = device[0].MX_INF_RS4852W;
					break;
				default:
					interface_num = device[0].MX_INF_RS232;
			}
			device[0].dsc_SetSerialInterface(portNo, interface_num);
			device[0].dsc_SaveAndRestart();
			device[0].dsc_Logout();
			device[0].dsc_Detach();

			dsc_SaveAndRestartRefresh();

		} catch (Exception e) {
			JOptionPane.showMessageDialog(null, "Set failed", "Error Message", JOptionPane.ERROR_MESSAGE);
			return;
		}
	}
	/**
	 * \brief	refresh opmode page.\n
	 * <b>detail description.</b>\n
	 *        refresh opmode page \n
	 */
	private void opmodeRefresh() {
		rightPage.removeAll();
		// init data
		JLabel empty1 = new JLabel("");
		JLabel empty2 = new JLabel("");
		JLabel empty3 = new JLabel("");
		JLabel title = new JLabel(" " + operationModesStr);
		JLabel mode  = new JLabel(" " + modeStr);

		// set label type
		setLabelType(title, 0);
		setLabelType(mode, 0);

		// add component
		containPage.weightx = 1;
		containPage.weighty = 0;
		addComponent(title, 0, 0, 3, 1);
		addComponent(mode, 1, 0, 1, 1);
		addComponent(opmodeInput, 1, 1, 1, 1);
		addComponent(empty1, 1, 2, 1, 1);
		addComponent(opmodePanel, 2, 0, 1, 1);

		containPage.weightx = 1;
		containPage.weighty = 1;
		addComponent(empty3, 3, 0, 3, 1);

		backIdx = OPMODE_PAGE;
		rightPage.validate();
		rightPage.updateUI();
	}
	/**
	 * \brief	submit opmode page.\n
	 * <b>detail description.</b>\n
	 *        submit opmode page and config device's settings about opmode \n
	 */
	private void opmodeSubmit() {
		int opmode_num = device[0].DISABLE_MODE;

		switch (opmodeInput.getSelectedIndex()) {
			case 0:
				opmode_num = device[0].DISABLE_MODE;
				break;
			case 1:
				opmode_num = device[0].DRV_MODE;
				break;
			case 2:
				opmode_num = device[0].RFC2217_MODE;
				break;
			case 3:
				opmode_num = device[0].TCPSER_MODE;
				break;
			case 4:
				opmode_num = device[0].TCPCLI_MODE;
				break;
			case 5:
				opmode_num = device[0].UDP_MODE;
				break;
			case 6:
				opmode_num = device[0].EMODEM_MODE;
				break;
			case 7:
				opmode_num = device[0].MCSC_MODE;
				break;
			default:
				opmode_num = device[0].DISABLE_MODE;
		}

		// set device configuration
		try {
			device[0].dsc_Attach();
			device[0].dsc_Login(password);
			device[0].dsc_SetOpModeEx(portNo, 0, opmode_num);
			device[0].dsc_SaveAndRestart();
			device[0].dsc_Logout();
			device[0].dsc_Detach();

			dsc_SaveAndRestartRefresh();

		} catch (Exception e) {
			JOptionPane.showMessageDialog(null, "Set failed", "Error Message", JOptionPane.ERROR_MESSAGE);
			return;
		}
	}
	/**
	 * \brief	refresh save and restart page.\n
	 * <b>detail description.</b>\n
	 *        refresh save and restart page \n
	 */
	private void dsc_SaveAndRestartRefresh() {
		rightPage.removeAll();
		// init data
		JLabel empty = new JLabel("");
		JLabel empty2 = new JLabel("");
		JLabel empty3 = new JLabel("");
		JLabel title = new JLabel(" Save Configuration OK!");
		JLabel msg1 = new JLabel(" Your configuration has been saved and your settings will take effect when the server is restarted.");
		JLabel msg2 = new JLabel(" Click on Restart to reboot the " + modelNameStr + " server now.");

		// set label type
		setLabelType(title, 0);
		setLabelType(msg1, 2);
		setLabelType(msg2, 2);

		// add component
		containPage.weightx = 1;
		containPage.weighty = 0;
		addComponent(title, 0, 0, 5, 1);
		addComponent(msg1, 1, 0, 5, 1);
		addComponent(msg2, 2, 0, 5, 1);
		addComponent(empty2, 3, 0, 1, 1);
		containPage.weightx = 0;
		containPage.weighty = 0;
		addComponent(back, 3, 1, 1, 1);
		addComponent(restart, 3, 2, 1, 1);
		addComponent(home, 3, 3, 1, 1);
		containPage.weightx = 1;
		containPage.weighty = 0;
		addComponent(empty3, 3, 4, 1, 1);
		containPage.weightx = 1;
		containPage.weighty = 1;
		addComponent(empty, 4, 0, 5, 1);

		rightPage.validate();
		rightPage.updateUI();
	}
	/**
	 * \brief	refresh the restart page.\n
	 * <b>detail description.</b>\n
	 *        refresh the restart page \n
	 * \param  msg           the message that be shown\n
	 */
	private void restartingRefresh(String msg) {
		rightPage.removeAll();
		// init data
		JLabel empty1 = new JLabel("");
		JLabel empty2 = new JLabel("");
		JLabel empty3 = new JLabel("");
		JLabel empty4 = new JLabel("");
		JLabel title = new JLabel("Restarting...");

		// set label type
		setLabelType(title, 0);

		homePanel.add(home);

		// add component
		containPage.weightx = 1;
		containPage.weighty = 0;
		addComponent(empty1, 0, 0, 1, 1);
		addComponent(title, 0, 1, 2, 1);
		addComponent(homePanel, 1, 1, 1, 1);

		containPage.weightx = 1;
		containPage.weighty = 1;
		addComponent(empty3, 2, 0, 3, 1);

		rightPage.validate();
		rightPage.updateUI();
	}
	/**
	 * \brief	refresh the tcp server example page.\n
	 * <b>detail description.</b>\n
	 *        refresh the tcp server example page \n
	 */
	private void tcpServerRefresh() {
		boolean connectOK = false;
		String blank = "       ";
		rightPage.removeAll();
		// init data
		localPort = Integer.parseInt(getParameter("ExLocalPort"));

		JLabel empty1 = new JLabel(blank + " Please set device's op mode to TCP Server mode, this is a client to connect device");
		JLabel empty2 = new JLabel("");

		readProgressBar.setVisible(true);
		writeProgressBar.setVisible(true);
		readProgressBar.setStringPainted(true);
		writeProgressBar.setStringPainted(true);
		writeProgressBar.setString("send " + 0 + " bytes");
		readProgressBar.setString("receive " + 0 + " bytes");

		setLabelType(empty1, 0);
		// go connect
		try {
			comConnect = new Socket(getMyIP(), localPort);
			connectOK = true;
		} catch (IOException ieo) {
			System.out.println("lost connection");
			System.out.println(ieo.toString());
		}

		// add component
		containPage.weightx = 1;
		containPage.weighty = 0;
		addComponent(empty1, 0, 0, 2, 1);
		addComponent(writeRpsPanel, 1, 0, 1, 1);
		addComponent(readRpsPanel, 1, 1, 1, 1);
		addComponent(writeContextScroll, 2, 0, 1, 1);
		addComponent(readContextScroll, 2, 1, 1, 1);
		addComponent(writeProgressBar, 3, 0, 1, 1);
		addComponent(readProgressBar, 3, 1, 1, 1);
		addComponent(writePanel, 4, 0, 1, 1);
		addComponent(readPanel, 4, 1, 1, 1);

		containPage.weightx = 1;
		containPage.weighty = 1;
		addComponent(empty2, 5, 0, 2, 1);

		rightPage.validate();
		rightPage.updateUI();

		if (connectOK) {
			// call thread
			(new drawThread()).start();
			(new readThread()).start();
			sentBut.setEnabled(true);
			repeatBut.setEnabled(true);
			clearBut.setEnabled(true);
		} else {
			sentBut.setEnabled(false);
			repeatBut.setEnabled(false);
			clearBut.setEnabled(false);
			JOptionPane.showMessageDialog(null, "Connect failed, Please set TCP Server mode and TCP port = " + localPort, "Results", JOptionPane.PLAIN_MESSAGE);
		}
	}

	/**
	 * \brief	close socket for example.\n
	 * <b>detail description.</b>\n
	 *        close socket for example \n
	 */
	private void closeTcpServer() {
		try {
			if (wThread != null)
				wThread.interrupt();
			wThread = null;
			if (bos != null)
				bos.close();
			if (bis != null)
				bis.close();
			if (comConnect != null)
				comConnect.close();
		} catch (IOException ieo) {
		}
	}
	/**
	 * \brief	send data for example.\n
	 * <b>detail description.</b>\n
	 *        send data for example \n
	 */
	private void sendDataTcpServer() {
		try {
			String send_data = writeContextInput.getText();
			bos = comConnect.getOutputStream();
			bos.write(send_data.getBytes());
			bos.flush();
			writeRpsPanel.addRps(send_data.length());
			writeProgressBar.setValue(send_data.length());
			writeProgressBar.setString
			("Send " + send_data.length() + " Bytes");
			writeContextInput.setText("");
		} catch (IOException ieo) {
		}
	}
	/**
	 * \brief	send pattern for example.\n
	 * <b>detail description.</b>\n
	 *        send pattern for example \n
	 */
	private void sendPatternTcpServer() {
		readProgressBar.setValue(0);
		writeProgressBar.setValue(0);
		writeProgressBar.setString("send " + 0 + " bytes");
		readProgressBar.setString("receive " + 0 + " bytes");
		if (repeatBut.getText().equals("Repeat")) {
			writeContextInput.setEnabled(false);
			sentBut.setEnabled(false);
			repeatBut.setText("Stop");

			wThread = new writeThread();
			wThread.start();
		} else {
			writeContextInput.setEnabled(true);
			sentBut.setEnabled(true);
			repeatBut.setText("Repeat");
			if (wThread != null)
				wThread.interrupt();
			wThread = null;
		}
	}
	/**
	 * \brief	do save and restart.\n
	 * <b>detail description.</b>\n
	 *        device would save settings and restart\n
	 */
	private void doSaveRestart() {
		try {
			device[0].dsc_Attach();
			device[0].dsc_Login(password);
			device[0].dsc_SaveAndRestart();
			device[0].dsc_Logout();
			device[0].dsc_Detach();
			restartingRefresh("Restarting...");

		} catch (Exception e) {
			//JOptionPane.showMessageDialog(null, "Set failed", "Error Message", JOptionPane.ERROR_MESSAGE);
			restartingRefresh("Restarting failed");
			return;
		}
		// for get new device name
		deviceNameInfo = null;
	}
	/**
	 * \brief	back to previous page.\n
	 * <b>detail description.</b>\n
	 *        go back the page that user browse before\n
	 */
	private void doBack() {
		switch (backIdx) {
			case OVERVIEW_PAGE:
				overviewRefresh();
				break;
			case NETWORK_PAGE:
				networkRefresh();
				break;
			case SERIAL_PAGE:
				serialRefresh();
				break;
			case OPMODE_PAGE:
				opmodeRefresh();
				break;
			default:
		}
	}
	/**
	 * \brief	add component into right page.\n
	 * <b>detail description.</b>\n
	 *        add component into right page\n
	 * \param  c             Component that want to add \n
	 * \param  row           which row that want to add into \n
	 * \param  column        which column that want to add into \n
	 * \param  width         Component's width \n
	 * \param  height        Component's height \n
	 */
	private void addComponent(Component c, int row, int column, int width, int height) {
		// set gridx and gridy
		containPage.gridx = column;
		containPage.gridy = row;

		// set gridwidth and gridheight
		containPage.gridwidth = width;
		containPage.gridheight = height;

		// set constraint
		gbLayout.setConstraints(c, containPage);
		rightPage.add(c);
	}
	/**
	 * \brief	check ip format.\n
	 * <b>detail description.</b>\n
	 *        check ip format invalid or not \n
	 *        this function is for set \n
	 * \param  st            ip address \n
	 * \retval true          the format is correct \n
	 * \retval false         the format is invalid \n
	 */
	private boolean isIp(String st) {
		String regex0 = "(2[0-4]\\d)" + "|(25[0-5])";
		String regex1 = "1\\d{2}";
		String regex2 = "[1-9]\\d";
		String regex3 = "\\d";
		String regex = "(" + regex0 + ")|(" + regex1 + ")|(" + regex2 + ")|("
		               + regex3 + ")";
		regex = "(" + regex + ").(" + regex + ").(" + regex + ").(" + regex
		        + ")";
		Pattern p = Pattern.compile(regex);
		Matcher m = p.matcher(st);
		return m.matches();
	}
	/**
	 * \brief	check ip format.\n
	 * <b>detail description.</b>\n
	 *        check ip format invalid or not \n
	 *        this function is for get \n
	 * \param  ip_format     ip address \n
	 * \retval String        ip address that after process\n
	 */
	private String checkIpFormat(String ip_format) {
		if (ip_format.equals("0.0.0.0")  || ip_format.equals("255.255.255.255") ||
		        ip_format.equals("/0.0.0.0") || ip_format.equals("/255.255.255.255"))
			return "";
		else
			return ip_format.replaceFirst("/", "");
	}
	/**
	 * \brief	the handle of tree class.\n
	 * <b>detail description.</b>\n
	 *        the handle of tree class \n
	 *        it would be call when user select tree node \n
	 * \param  selPath     Tree Selection \n
	 */
	public void clickTreeView(TreePath selPath) {
		closeTcpServer();
		getDeviceInfo();

		if (selPath.toString().equals("[" + mainMenuStr + ", " + overviewStr + "]"))
			overviewRefresh();
		else if (selPath.toString().equals("[" + mainMenuStr + ", " + exampleStr + ", " + tcpServerStr + "]"))
			tcpServerRefresh();
		else if (selPath.toString().equals("[" + mainMenuStr + ", " + basicSettingsStr + ", " + networkSettingsStr + "]"))
			networkRefresh();
		else if (selPath.toString().equals("[" + mainMenuStr + ", " + basicSettingsStr + ", " + serialPortSettingsStr + "]"))
			serialRefresh();
		else if (selPath.toString().equals("[" + mainMenuStr + ", " + basicSettingsStr + ", " + operationModesStr + "]"))
			opmodeRefresh();
		else if (selPath.toString().equals("[" + mainMenuStr + ", " + saveNRestartStr + "]"))
			dsc_SaveAndRestartRefresh();
		rightPage.updateUI();
	}
	/**
	 * \brief	set label string type.\n
	 * <b>detail description.</b>\n
	 *        set label string type \n
	 * \param  label    set_label \n
	 * \param  type     string type \n
	 */
	private void setLabelType(JLabel set_label, int type) {
		switch (type) {
				// page title
			case 0:
				set_label.setFont(new Font(fontStr, Font.BOLD, 12));
				set_label.setForeground(new Color(0x00, 0x99, 0x66));
				break;
				// page sub title
			case 1:
				set_label.setFont(new Font(fontStr, Font.BOLD, 12));
				set_label.setForeground(new Color(0x00, 0x99, 0x66));
				break;
				// page item title
			case 2:
				set_label.setFont(new Font(fontStr, Font.PLAIN, 11));
				set_label.setForeground(new Color(0x00, 0x00, 0x00));
				break;
				// block item
			case 3:
				set_label.setFont(new Font(fontStr, Font.PLAIN, 12));
				set_label.setForeground(new Color(0x00, 0x00, 0x00));
				set_label.setOpaque(true);
				set_label.setBackground(new Color(0xcc, 0xf4, 0xde));
				break;
			case 4:
				set_label.setOpaque(true);
				set_label.setBackground(new Color(0xf5, 0xf5, 0xdc));
				break;
			case 5:
				set_label.setOpaque(true);
				set_label.setBackground(new Color(0xf0, 0xff, 0xff));
				break;
			case 6:
				set_label.setFont(new Font(fontStr, Font.BOLD, 12));
				set_label.setForeground(new Color(0x00, 0x00, 0x00));
				set_label.setOpaque(true);
				set_label.setBackground(new Color(0xcc, 0xf4, 0xde));
				break;
			default:
				break;
		}
	}
	/**
	 * \brief	build the tree's architecture.\n
	 * <b>detail description.</b>\n
	 *        build the tree's architecture \n
	 * \retval TreeNode        the tree node\n
	 */
	private TreeNode makeTree() {
		DefaultMutableTreeNode root     = new DefaultMutableTreeNode(mainMenuStr, true);
		DefaultMutableTreeNode overview = new DefaultMutableTreeNode(overviewStr, false);
		DefaultMutableTreeNode basic    = new DefaultMutableTreeNode(basicSettingsStr, true);
		DefaultMutableTreeNode example  = new DefaultMutableTreeNode(exampleStr, true);
		DefaultMutableTreeNode save_rst = new DefaultMutableTreeNode(saveNRestartStr, false);

		basic.add(new DefaultMutableTreeNode(networkSettingsStr, false));
		basic.add(new DefaultMutableTreeNode(serialPortSettingsStr, false));
		basic.add(new DefaultMutableTreeNode(operationModesStr, false));

		example.add(new DefaultMutableTreeNode(tcpServerStr, false));

		root.add(overview);
		root.add(basic);
		if (TCPSVREXAMPLE)
			root.add(example);
		root.add(save_rst);
		return root;
	}
	/**
	 * \brief	get local host ip.\n
	 * <b>detail description.</b>\n
	 *        get local host ip \n
	 * \retval InetAddress        local host ip\n
	 * \retval null               get local host ip failed\n
	 */
	private InetAddress getMyIP() {
		InetAddress myIPaddress = null;
		try {
			URL url = this.getCodeBase();
			myIPaddress = InetAddress.getByName(url.getHost());
			return (myIPaddress);
		} catch (UnknownHostException e) {
			return null;
		}

	}
	/**
	 * \brief	set model name.\n
	 * <b>detail description.</b>\n
	 *        set model name \n
	 */
	private void setModelName() {
		String baudRate_tmp_str[] = { "50", "75", "110", "134", "150", "300", "600",
		                              "1200", "1800", "2400", "4800", "7200", "9600",
		                              "19200", "38400", "57600", "115200", "230400",
		                              "460800", "921600", "Other"
		                            };

		Device.DeviceInfo deviceInfo = null;
		modelNameStr = "Unknown Model";
		modelNo = UNKNOWN_MODEL;

		deviceInfo = device[0].getDeviceInfo();
		if (deviceInfo != null) {
			for (int i = 0; i < modelNameIdx.length; i++) {
				if (deviceInfo.apId == modelNameIdx[i][0] &&
				        deviceInfo.hwId == modelNameIdx[i][1]) {
					modelNo = i + 1;
					modelNameStr = modelName[i];

					// MiiNePort E2-H support baud rate 921600
					if (modelNo == MIINEPORT_E2H) {
						baudRateStr = new String[baudRate_tmp_str.length];
						opmodeInput = new JComboBox(opmodeStrE2);
					}
					else if (modelNo == MIINEPORT_E2) {
						baudRateStr = new String[baudRate_tmp_str.length-2];
						opmodeInput = new JComboBox(opmodeStrE2);
					}
					else if (modelNo == MIINEPORT_E3H) {
						baudRateStr = new String[baudRate_tmp_str.length];
						opmodeInput = new JComboBox(opmodeStrE3);
					}
					else if (modelNo == MIINEPORT_E3) {
						baudRateStr = new String[baudRate_tmp_str.length-2];
						opmodeInput = new JComboBox(opmodeStrE3);
					}

				}
			}
		}

		if (baudRateStr == null)
			baudRateStr = new String[baudRate_tmp_str.length-2];

		for (int j = 0; j < baudRateStr.length - 1; j++)
			baudRateStr[j] = baudRate_tmp_str[j];
		baudRateStr[baudRateStr.length-1] = baudRate_tmp_str[baudRate_tmp_str.length-1];
	}

	//Returns an ImageIcon, or null if the path was invalid.
	//When running an applet using Java Plug-in,
	//getResourceAsStream is more efficient than getResource.
	/**
	 * \brief	create Applet ImageIcon.\n
	 * <b>detail description.</b>\n
	 *        create icon for applet \n
	 * \param  path             path of icon \n
	 * \param  description     	description of icon \n
	 * \retval ImageIcon        image icon\n
	 */
	public static ImageIcon createAppletImageIcon(String path, String description) {
		int MAX_IMAGE_SIZE = 75000; //Change this to the size of your biggest image, in bytes.

		int count = 0;
		int off = 0;
		BufferedInputStream imgStream = new BufferedInputStream(EZPage.class.getResourceAsStream(path));
		if (imgStream != null) {
			byte buf[] = new byte[MAX_IMAGE_SIZE];
			try {
				while ((count = imgStream.read(buf, off, MAX_IMAGE_SIZE - off)) >= 0) {
					off += count;
				}
			} catch (IOException ieo) {
				System.err.println("Couldn't read stream from file: " + path);
			}

			try {
				imgStream.close();
			} catch (IOException ieo) {
				System.err.println("Can't close file " + path);
			}

			if (count == 0) {
				System.err.println("Empty file: " + path);
				return null;
			}
			return new ImageIcon(Toolkit.getDefaultToolkit().createImage(buf),
			                     description);
		} else {
			System.err.println("Couldn't find file: " + path);
			return null;
		}
	}
}
